// Code generated by interfacer; DO NOT EDIT

package hcapi2

import (
	"context"
	"github.com/hetznercloud/hcloud-go/v2/hcloud"
)

// CertificateClientBase is an interface generated for "github.com/hetznercloud/hcloud-go/v2/hcloud.CertificateClient".
type CertificateClientBase interface {
	All(context.Context) ([]*hcloud.Certificate, error)
	AllWithOpts(context.Context, hcloud.CertificateListOpts) ([]*hcloud.Certificate, error)
	Create(context.Context, hcloud.CertificateCreateOpts) (*hcloud.Certificate, *hcloud.Response, error)
	CreateCertificate(context.Context, hcloud.CertificateCreateOpts) (hcloud.CertificateCreateResult, *hcloud.Response, error)
	Delete(context.Context, *hcloud.Certificate) (*hcloud.Response, error)
	Get(context.Context, string) (*hcloud.Certificate, *hcloud.Response, error)
	GetByID(context.Context, int64) (*hcloud.Certificate, *hcloud.Response, error)
	GetByName(context.Context, string) (*hcloud.Certificate, *hcloud.Response, error)
	List(context.Context, hcloud.CertificateListOpts) ([]*hcloud.Certificate, *hcloud.Response, error)
	RetryIssuance(context.Context, *hcloud.Certificate) (*hcloud.Action, *hcloud.Response, error)
	Update(context.Context, *hcloud.Certificate, hcloud.CertificateUpdateOpts) (*hcloud.Certificate, *hcloud.Response, error)
}
