// Code generated by MockGen. DO NOT EDIT.
// Source: github.com/hetznercloud/cli/internal/hcapi2 (interfaces: LoadBalancerTypeClient)

// Package hcapi2_mock is a generated GoMock package.
package hcapi2_mock

import (
	context "context"
	reflect "reflect"

	gomock "github.com/golang/mock/gomock"
	hcloud "github.com/hetznercloud/hcloud-go/v2/hcloud"
)

// MockLoadBalancerTypeClient is a mock of LoadBalancerTypeClient interface.
type MockLoadBalancerTypeClient struct {
	ctrl     *gomock.Controller
	recorder *MockLoadBalancerTypeClientMockRecorder
}

// MockLoadBalancerTypeClientMockRecorder is the mock recorder for MockLoadBalancerTypeClient.
type MockLoadBalancerTypeClientMockRecorder struct {
	mock *MockLoadBalancerTypeClient
}

// NewMockLoadBalancerTypeClient creates a new mock instance.
func NewMockLoadBalancerTypeClient(ctrl *gomock.Controller) *MockLoadBalancerTypeClient {
	mock := &MockLoadBalancerTypeClient{ctrl: ctrl}
	mock.recorder = &MockLoadBalancerTypeClientMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockLoadBalancerTypeClient) EXPECT() *MockLoadBalancerTypeClientMockRecorder {
	return m.recorder
}

// All mocks base method.
func (m *MockLoadBalancerTypeClient) All(arg0 context.Context) ([]*hcloud.LoadBalancerType, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "All", arg0)
	ret0, _ := ret[0].([]*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// All indicates an expected call of All.
func (mr *MockLoadBalancerTypeClientMockRecorder) All(arg0 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "All", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).All), arg0)
}

// AllWithOpts mocks base method.
func (m *MockLoadBalancerTypeClient) AllWithOpts(arg0 context.Context, arg1 hcloud.LoadBalancerTypeListOpts) ([]*hcloud.LoadBalancerType, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "AllWithOpts", arg0, arg1)
	ret0, _ := ret[0].([]*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// AllWithOpts indicates an expected call of AllWithOpts.
func (mr *MockLoadBalancerTypeClientMockRecorder) AllWithOpts(arg0, arg1 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "AllWithOpts", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).AllWithOpts), arg0, arg1)
}

// Get mocks base method.
func (m *MockLoadBalancerTypeClient) Get(arg0 context.Context, arg1 string) (*hcloud.LoadBalancerType, *hcloud.Response, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Get", arg0, arg1)
	ret0, _ := ret[0].(*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(*hcloud.Response)
	ret2, _ := ret[2].(error)
	return ret0, ret1, ret2
}

// Get indicates an expected call of Get.
func (mr *MockLoadBalancerTypeClientMockRecorder) Get(arg0, arg1 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Get", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).Get), arg0, arg1)
}

// GetByID mocks base method.
func (m *MockLoadBalancerTypeClient) GetByID(arg0 context.Context, arg1 int64) (*hcloud.LoadBalancerType, *hcloud.Response, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "GetByID", arg0, arg1)
	ret0, _ := ret[0].(*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(*hcloud.Response)
	ret2, _ := ret[2].(error)
	return ret0, ret1, ret2
}

// GetByID indicates an expected call of GetByID.
func (mr *MockLoadBalancerTypeClientMockRecorder) GetByID(arg0, arg1 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "GetByID", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).GetByID), arg0, arg1)
}

// GetByName mocks base method.
func (m *MockLoadBalancerTypeClient) GetByName(arg0 context.Context, arg1 string) (*hcloud.LoadBalancerType, *hcloud.Response, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "GetByName", arg0, arg1)
	ret0, _ := ret[0].(*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(*hcloud.Response)
	ret2, _ := ret[2].(error)
	return ret0, ret1, ret2
}

// GetByName indicates an expected call of GetByName.
func (mr *MockLoadBalancerTypeClientMockRecorder) GetByName(arg0, arg1 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "GetByName", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).GetByName), arg0, arg1)
}

// List mocks base method.
func (m *MockLoadBalancerTypeClient) List(arg0 context.Context, arg1 hcloud.LoadBalancerTypeListOpts) ([]*hcloud.LoadBalancerType, *hcloud.Response, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "List", arg0, arg1)
	ret0, _ := ret[0].([]*hcloud.LoadBalancerType)
	ret1, _ := ret[1].(*hcloud.Response)
	ret2, _ := ret[2].(error)
	return ret0, ret1, ret2
}

// List indicates an expected call of List.
func (mr *MockLoadBalancerTypeClientMockRecorder) List(arg0, arg1 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "List", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).List), arg0, arg1)
}

// LoadBalancerTypeDescription mocks base method.
func (m *MockLoadBalancerTypeClient) LoadBalancerTypeDescription(arg0 int64) string {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "LoadBalancerTypeDescription", arg0)
	ret0, _ := ret[0].(string)
	return ret0
}

// LoadBalancerTypeDescription indicates an expected call of LoadBalancerTypeDescription.
func (mr *MockLoadBalancerTypeClientMockRecorder) LoadBalancerTypeDescription(arg0 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "LoadBalancerTypeDescription", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).LoadBalancerTypeDescription), arg0)
}

// LoadBalancerTypeName mocks base method.
func (m *MockLoadBalancerTypeClient) LoadBalancerTypeName(arg0 int64) string {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "LoadBalancerTypeName", arg0)
	ret0, _ := ret[0].(string)
	return ret0
}

// LoadBalancerTypeName indicates an expected call of LoadBalancerTypeName.
func (mr *MockLoadBalancerTypeClientMockRecorder) LoadBalancerTypeName(arg0 interface{}) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "LoadBalancerTypeName", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).LoadBalancerTypeName), arg0)
}

// Names mocks base method.
func (m *MockLoadBalancerTypeClient) Names() []string {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "Names")
	ret0, _ := ret[0].([]string)
	return ret0
}

// Names indicates an expected call of Names.
func (mr *MockLoadBalancerTypeClientMockRecorder) Names() *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "Names", reflect.TypeOf((*MockLoadBalancerTypeClient)(nil).Names))
}
