// Code generated by interfacer; DO NOT EDIT

package hcapi2

import (
	"context"
	"github.com/hetznercloud/hcloud-go/v2/hcloud"
)

// FloatingIPClientBase is an interface generated for "github.com/hetznercloud/hcloud-go/v2/hcloud.FloatingIPClient".
type FloatingIPClientBase interface {
	All(context.Context) ([]*hcloud.FloatingIP, error)
	AllWithOpts(context.Context, hcloud.FloatingIPListOpts) ([]*hcloud.FloatingIP, error)
	Assign(context.Context, *hcloud.FloatingIP, *hcloud.Server) (*hcloud.Action, *hcloud.Response, error)
	ChangeDNSPtr(context.Context, *hcloud.FloatingIP, string, *string) (*hcloud.Action, *hcloud.Response, error)
	ChangeProtection(context.Context, *hcloud.FloatingIP, hcloud.FloatingIPChangeProtectionOpts) (*hcloud.Action, *hcloud.Response, error)
	Create(context.Context, hcloud.FloatingIPCreateOpts) (hcloud.FloatingIPCreateResult, *hcloud.Response, error)
	Delete(context.Context, *hcloud.FloatingIP) (*hcloud.Response, error)
	Get(context.Context, string) (*hcloud.FloatingIP, *hcloud.Response, error)
	GetByID(context.Context, int64) (*hcloud.FloatingIP, *hcloud.Response, error)
	GetByName(context.Context, string) (*hcloud.FloatingIP, *hcloud.Response, error)
	List(context.Context, hcloud.FloatingIPListOpts) ([]*hcloud.FloatingIP, *hcloud.Response, error)
	Unassign(context.Context, *hcloud.FloatingIP) (*hcloud.Action, *hcloud.Response, error)
	Update(context.Context, *hcloud.FloatingIP, hcloud.FloatingIPUpdateOpts) (*hcloud.FloatingIP, *hcloud.Response, error)
}
